@@ -44,17 +44,20 @@ import sys
 # Some general utility functions
 def _createTablesString(tablesList):
     """Creates a tables string for liblouis calls"""
-    return b",".join([x.encode("ASCII") if isinstance(x, str) else bytes(x) for x in tablesList])
+    return b",".join([x.encode(filesystemencoding) if isinstance(x, str) else bytes(x) for x in tablesList])
 
 def _createTypeformbuf(length, typeform=None):
     """Creates a typeform buffer for liblouis calls"""
     return (c_ushort*length)(*typeform) if typeform else (c_ushort*length)()
 
-createEncodedByeString = None
+ENCODING_ERROR_HANDLER = None
+createEncodedByteString = None
 if sys.version_info.major == 2:
-    createEncodedByeString = lambda x: unicode(x).encode(conversionEncoding)
+    ENCODING_ERROR_HANDLER = "strict"
+    createEncodedByteString = lambda x: unicode(x).encode(conversionEncoding, errors=ENCODING_ERROR_HANDLER)
 else: # sys.version_info[0] == 3
-    createEncodedByeString = lambda x: str(x).encode(conversionEncoding)
+    ENCODING_ERROR_HANDLER = "surrogatepass"
+    createEncodedByteString = lambda x: str(x).encode(conversionEncoding, errors=ENCODING_ERROR_HANDLER)
 
 try:
     # Native win32
@@ -64,7 +67,7 @@ except NameError:
     # Unix/Cygwin
     _loader = cdll
     _functype = CFUNCTYPE
-liblouis = _loader["liblouis.so.17"]
+liblouis = _loader["liblouis.so.20"]
 
 atexit.register(liblouis.lou_free)
 
@@ -107,10 +110,9 @@ liblouis.lou_charToDots.argtypes = (
     c_char_p, POINTER(c_char), POINTER(c_char), 
          c_int, c_int)
 
-logcallback = _functype(None, c_int, c_char_p)
+LogCallback = _functype(None, c_int, c_char_p)
 
 liblouis.lou_registerLogCallback.restype = None
-liblouis.lou_registerLogCallback.argtypes = (logcallback,)
 
 liblouis.lou_setLogLevel.restype = None
 liblouis.lou_setLogLevel.argtypes = (c_int,)
@@ -127,6 +129,9 @@ wideCharBytes = liblouis.lou_charSize()
 # This default will handle the case where every input character is
 # undefined in the translation table.
 outlenMultiplier = 4 + wideCharBytes * 2
+#: Specifies the encoding to use when encode/decode filenames
+#: @type: str
+filesystemencoding = sys.getfilesystemencoding()
 #: Specifies the encoding to use when converting from byte strings to unicode strings.
 #: @type: str
 conversionEncoding = "utf_%d_le" % (wideCharBytes * 8)
@@ -169,7 +174,7 @@ def translate(tableList, inbuf, typeform=None, cursorPos=0, mode=0):
     @see: lou_translate in the liblouis documentation
     """
     tablesString = _createTablesString(tableList)
-    inbuf = createEncodedByeString(inbuf)
+    inbuf = createEncodedByteString(inbuf)
     inlen = c_int(len(inbuf) // wideCharBytes)
     outlen = c_int(inlen.value*outlenMultiplier)
     outbuf = create_string_buffer(outlen.value * wideCharBytes)
@@ -185,7 +190,7 @@ def translate(tableList, inbuf, typeform=None, cursorPos=0, mode=0):
         raise RuntimeError("Can't translate: tables %s, inbuf %s, typeform %s, cursorPos %s, mode %s"%(tableList, inbuf, typeform, cursorPos, mode))
     if isinstance(typeform, list):
         typeform[:] = list(typeformbuf)
-    return outbuf.raw[:outlen.value * wideCharBytes].decode(conversionEncoding), inPos[:outlen.value], outPos[:inlen.value], cursorPos.value
+    return outbuf.raw[:outlen.value * wideCharBytes].decode(conversionEncoding, errors=ENCODING_ERROR_HANDLER), inPos[:outlen.value], outPos[:inlen.value], cursorPos.value
 
 def translateString(tableList, inbuf, typeform=None, mode=0):
     """Translate a string of characters.
@@ -204,7 +209,7 @@ def translateString(tableList, inbuf, typeform=None, mode=0):
     @see: lou_translateString in the liblouis documentation
     """
     tablesString = _createTablesString(tableList)
-    inbuf = createEncodedByeString(inbuf)
+    inbuf = createEncodedByteString(inbuf)
     inlen = c_int(len(inbuf) // wideCharBytes)
     outlen = c_int(inlen.value*outlenMultiplier)
     outbuf = create_string_buffer(outlen.value * wideCharBytes)
@@ -217,7 +222,7 @@ def translateString(tableList, inbuf, typeform=None, mode=0):
         raise RuntimeError("Can't translate: tables %s, inbuf %s, typeform %s, mode %s"%(tableList, inbuf, typeform, mode))
     if isinstance(typeform, list):
         typeform[:] = list(typeformbuf)
-    return outbuf.raw[:outlen.value * wideCharBytes].decode(conversionEncoding)
+    return outbuf.raw[:outlen.value * wideCharBytes].decode(conversionEncoding, errors=ENCODING_ERROR_HANDLER)
 
 def backTranslate(tableList, inbuf, typeform=None, cursorPos=0, mode=0):
     """Back translates a string of characters, providing position information.
@@ -240,7 +245,7 @@ def backTranslate(tableList, inbuf, typeform=None, cursorPos=0, mode=0):
     @see: lou_backTranslate in the liblouis documentation.
     """
     tablesString = _createTablesString(tableList)
-    inbuf = createEncodedByeString(inbuf)
+    inbuf = createEncodedByteString(inbuf)
     inlen = c_int(len(inbuf) // wideCharBytes)
     outlen = c_int(inlen.value * outlenMultiplier)
     outbuf = create_string_buffer(outlen.value * wideCharBytes)
@@ -256,7 +261,7 @@ def backTranslate(tableList, inbuf, typeform=None, cursorPos=0, mode=0):
         raise RuntimeError("Can't back translate: tables %s, inbuf %s, typeform %s, cursorPos %d, mode %d"%(tableList, inbuf, typeform, cursorPos, mode))
     if isinstance(typeform, list):
         typeform[:] = list(typeformbuf)
-    return outbuf.raw[:outlen.value * wideCharBytes].decode(conversionEncoding), inPos[:outlen.value], outPos[:inlen.value], cursorPos.value
+    return outbuf.raw[:outlen.value * wideCharBytes].decode(conversionEncoding, errors=ENCODING_ERROR_HANDLER), inPos[:outlen.value], outPos[:inlen.value], cursorPos.value
 
 def backTranslateString(tableList, inbuf, typeform=None, mode=0):
     """Back translate from Braille.
@@ -275,7 +280,7 @@ def backTranslateString(tableList, inbuf, typeform=None, mode=0):
     @see: lou_backTranslateString in the liblouis documentation.
     """
     tablesString = _createTablesString(tableList)
-    inbuf = createEncodedByeString(inbuf)
+    inbuf = createEncodedByteString(inbuf)
     inlen = c_int(len(inbuf) // wideCharBytes)
     outlen = c_int(inlen.value * outlenMultiplier)
     outbuf = create_string_buffer(outlen.value * wideCharBytes)
@@ -287,7 +292,7 @@ def backTranslateString(tableList, inbuf, typeform=None, mode=0):
         raise RuntimeError("Can't back translate: tables %s, inbuf %s, mode %d"%(tableList, inbuf, mode))
     if isinstance(typeform, list):
         typeform[:] = list(typeformbuf)
-    return outbuf.raw[:outlen.value * wideCharBytes].decode(conversionEncoding)
+    return outbuf.raw[:outlen.value * wideCharBytes].decode(conversionEncoding, errors=ENCODING_ERROR_HANDLER)
 
 def hyphenate(tableList, inbuf, mode=0):
     """Get information for hyphenation.
@@ -308,7 +313,7 @@ def hyphenate(tableList, inbuf, mode=0):
     @see: lou_hyphenate in the liblouis documentation.
     """
     tablesString = _createTablesString(tableList)
-    inbuf = createEncodedByeString(inbuf)
+    inbuf = createEncodedByteString(inbuf)
     inlen = c_int(len(inbuf) // wideCharBytes)
     hyphen_string = create_string_buffer(inlen.value + 1) 
     if not liblouis.lou_hyphenate(tablesString, inbuf, inlen, hyphen_string, mode):
@@ -364,12 +369,12 @@ def dotsToChar(tableList, inbuf):
     @see: lou_dotsToChar in the liblouis documentation
     """
     tablesString = _createTablesString(tableList)
-    inbuf = createEncodedByeString(inbuf)
+    inbuf = createEncodedByteString(inbuf)
     length = c_int(len(inbuf) // wideCharBytes)
     outbuf = create_string_buffer(length.value * wideCharBytes)
     if not liblouis.lou_dotsToChar(tablesString, inbuf, outbuf, length, 0):
         raise RuntimeError("Can't convert dots to char: tables %s, inbuf %s"%(tableList, inbuf))
-    return outbuf.raw[:length.value * wideCharBytes].decode(conversionEncoding)
+    return outbuf.raw[:length.value * wideCharBytes].decode(conversionEncoding, errors=ENCODING_ERROR_HANDLER)
 
 def charToDots(tableList, inbuf, mode=0):
     """"Convert a string of characterss to a string of dot patterns according to the specifications in tableList.
@@ -383,17 +388,17 @@ def charToDots(tableList, inbuf, mode=0):
     @see: lou_charToDOts in the liblouis documentation
     """
     tablesString = _createTablesString(tableList)
-    inbuf = createEncodedByeString(inbuf)
+    inbuf = createEncodedByteString(inbuf)
     length = c_int(len(inbuf) // wideCharBytes)
     outbuf = create_string_buffer(length.value * wideCharBytes)
     if not liblouis.lou_charToDots(tablesString, inbuf, outbuf, length, mode):
         raise RuntimeError("Can't convert char to dots: tables %s, inbuf %s, mode %d"%(tableList, inbuf, mode))
-    return outbuf.raw[:length.value * wideCharBytes].decode(conversionEncoding)
+    return outbuf.raw[:length.value * wideCharBytes].decode(conversionEncoding, errors=ENCODING_ERROR_HANDLER)
 
-def registerLogCallback(logcallback):
+def registerLogCallback(logCallback):
     """Register logging callbacks.
     Set to C{None} for default callback.
-    @param logcallback: The callback to use.
+    @param logCallback: The callback to use.
         The callback must take two arguments:
         @param level: The log level on which a message is logged.
         @type level: int
@@ -403,13 +408,15 @@ def registerLogCallback(logcallback):
 
         Example callback:
 
-        @louis.logcallback
+        @louis.LogCallback
         def incomingLouisLog(level, message):
             print("Message %s logged at level %d" % (message.decode("ASCII"), level))
 
-    @type logcallback: L{logcallback}
+    @type logCallback: L{LogCallback}
     """
-    return liblouis.lou_registerLogCallback(logcallback)
+    if logCallback is not None and not isinstance(logCallback, LogCallback):
+        raise TypeError("logCallback should be of type {} or NoneType".format(LogCallback.__name__))
+    return liblouis.lou_registerLogCallback(logCallback)
 
 def setLogLevel(level):
 	"""Set the level for logging callback to be called at.
@@ -444,7 +451,8 @@ compbrlAtCursor = 2
 dotsIO = 4
 compbrlLeftCursor = 32
 ucBrl = 64
-noUndefinedDots = 128
+noUndefined = 128
+noUndefinedDots = noUndefined # alias for backward compatiblity
 partialTrans = 256
 #}
 
